@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-search.c,v 1.2 2005/12/12 17:27:55 pborelli Exp $
+ * $Id: gedit-commands-search.c,v 1.4 2006/01/03 14:54:22 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -159,17 +159,24 @@ static void
 phrase_found (GeditWindow *window,
 	      gint         occurrences)
 {
-	if (occurrences > 0)
+	if (occurrences > 1)
 		gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
 					       window->priv->generic_message_cid,
-					       ngettext("Found and replaced %d occurrence.",
-					     	        "Found and replaced %d occurrences.",
+					       ngettext("Found and replaced %d occurrence",
+					     	        "Found and replaced %d occurrences",
 					     	        occurrences),
 					       occurrences);
-	else
-		gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
-					       window->priv->generic_message_cid,
-					       " ");
+	else 
+	{
+		if (occurrences == 1)
+			gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
+						       window->priv->generic_message_cid,
+						       _("Found and replaced one occurrence"));
+		else
+			gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
+						       window->priv->generic_message_cid,
+						       " ");
+	}
 }
 
 static void
@@ -311,9 +318,6 @@ do_find (GeditSearchDialog *dialog,
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
 					   GEDIT_SEARCH_DIALOG_REPLACE_RESPONSE,
 					   found);
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
-					   GEDIT_SEARCH_DIALOG_REPLACE_ALL_RESPONSE,
-					   found);
 
 	restore_last_searched_data (dialog, doc);
 }
@@ -469,9 +473,6 @@ do_replace_all (GeditSearchDialog *dialog,
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
 					   GEDIT_SEARCH_DIALOG_REPLACE_RESPONSE,
 					   FALSE);
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
-					   GEDIT_SEARCH_DIALOG_REPLACE_ALL_RESPONSE,
-					   FALSE);
 
 	restore_last_searched_data (dialog, doc);
 }
