@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.c,v 1.69 2005/12/29 15:13:39 pborelli Exp $
+ * $Id: gedit-utils.c,v 1.73 2006/01/15 08:54:26 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -280,7 +280,7 @@ gedit_utils_uri_exists (const gchar* text_uri)
 		
 	g_return_val_if_fail (text_uri != NULL, FALSE);
 	
-	gedit_debug_message (DEBUG_FILE, "text_uri: %s", text_uri);
+	gedit_debug_message (DEBUG_UTILS, "text_uri: %s", text_uri);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	g_return_val_if_fail (uri != NULL, FALSE);
@@ -289,7 +289,7 @@ gedit_utils_uri_exists (const gchar* text_uri)
 
 	gnome_vfs_uri_unref (uri);
 
-	gedit_debug_message (DEBUG_FILE, res ? "TRUE" : "FALSE");
+	gedit_debug_message (DEBUG_UTILS, res ? "TRUE" : "FALSE");
 
 	return res;
 }
@@ -744,29 +744,33 @@ gedit_utils_replace_home_dir_with_tilde (const gchar *uri)
 guint
 gedit_utils_get_current_workspace (GdkScreen *screen)
 {
-       GdkWindow *root_win = gdk_screen_get_root_window (screen);
-       GdkDisplay *display = gdk_screen_get_display (screen);
+	GdkWindow *root_win;
+	GdkDisplay *display;
+	Atom type;
+	gint format;
+	gulong nitems;
+	gulong bytes_after;
+	guint *current_desktop;
+	guint ret = 0;
 
-       Atom type;
-       gint format;
-       gulong nitems;
-       gulong bytes_after;
-       guint *current_desktop;
-       guint ret = 0;
+	g_return_val_if_fail (GDK_IS_SCREEN (screen), 0);
 
-       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
-                           gdk_x11_get_xatom_by_name_for_display (display, "_NET_CURRENT_DESKTOP"),
-                           0, G_MAXLONG,
-                           False, XA_CARDINAL, &type, &format, &nitems,
-                           &bytes_after, (gpointer)&current_desktop);
+	root_win = gdk_screen_get_root_window (screen);
+	display = gdk_screen_get_display (screen);
 
-       if (type == XA_CARDINAL && format == 32 && nitems > 0)
-       {
-               ret = current_desktop[0];
-               XFree (current_desktop);
-       }
+	XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
+			    gdk_x11_get_xatom_by_name_for_display (display, "_NET_CURRENT_DESKTOP"),
+			    0, G_MAXLONG,
+			    False, XA_CARDINAL, &type, &format, &nitems,
+			    &bytes_after, (gpointer)&current_desktop);
 
-       return ret;
+	if (type == XA_CARDINAL && format == 32 && nitems > 0)
+	{
+		ret = current_desktop[0];
+		XFree (current_desktop);
+	}
+
+	return ret;
 }
 
 /**
@@ -779,31 +783,37 @@ gedit_utils_get_current_workspace (GdkScreen *screen)
 guint
 gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 {
-       GdkWindow *window = GTK_WIDGET (gtkwindow)->window;
-       GdkDisplay *display = gdk_drawable_get_display (window);
+	GdkWindow *window;
+	GdkDisplay *display;
+	Atom type;
+	gint format;
+	gulong nitems;
+	gulong bytes_after;
+	guint *workspace;
+	guint ret = GEDIT_ALL_WORKSPACES;
 
-       Atom type;
-       gint format;
-       gulong nitems;
-       gulong bytes_after;
-       guint *workspace;
-       guint ret = GEDIT_ALL_WORKSPACES;
+	g_return_val_if_fail (GTK_IS_WINDOW (gtkwindow), 0);
+	g_return_val_if_fail (GTK_WIDGET_REALIZED (GTK_WIDGET (gtkwindow)), 0);
 
-       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (window),
-                           gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_DESKTOP"),
-                           0, G_MAXLONG,
-                           False, XA_CARDINAL, &type, &format, &nitems,
-                           &bytes_after, (gpointer)&workspace);
+	window = GTK_WIDGET (gtkwindow)->window;
+	display = gdk_drawable_get_display (window);
 
-       if (type == XA_CARDINAL && format == 32 && nitems > 0)
-       {
-               ret = workspace[0];
-               XFree (workspace);
-       }
+	XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (window),
+			    gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_DESKTOP"),
+			    0, G_MAXLONG,
+			    False, XA_CARDINAL, &type, &format, &nitems,
+			    &bytes_after, (gpointer)&workspace);
 
-       return ret;
+	if (type == XA_CARDINAL && format == 32 && nitems > 0)
+	{
+		ret = workspace[0];
+		XFree (workspace);
+	}
+
+	return ret;
 }
-void  
+
+void
 gedit_utils_activate_url (GtkAboutDialog *about,
 			  const gchar    *url,
 			  gpointer        data)
@@ -847,7 +857,7 @@ gedit_utils_is_valid_uri (const gchar *uri)
 		return FALSE;
 
 	/* We expect to have a fully valid set of characters */
-	for (p = uri; *p; p++) {
+	for (p = (const guchar *)uri; *p; p++) {
 		if (*p == '%')
 		{
 			++p;
@@ -869,10 +879,6 @@ gedit_utils_is_valid_uri (const gchar *uri)
 }
 
 
-
-#define GEDIT_MISSING_FILE    N_("<span size=\"large\" weight=\"bold\">Unable to find file <i>%s</i>.</span>\n\nPlease, check your installation.")
-#define GEDIT_MISSING_WIDGETS N_("<span size=\"large\" weight=\"bold\">Unable to find the required widgets inside file <i>%s</i>..</span>\n\nPlease, check your installation.")
-
 /**
  * gedit_utils_get_glade_widgets:
  * @filename: the path to the glade file
@@ -900,6 +906,8 @@ gedit_utils_get_glade_widgets (const gchar *filename,
 	va_list args;
 	const gchar *name;
 	gchar *msg;
+	gchar *filename_markup;
+	gchar *msg_plain;
 	gboolean ret = TRUE;
 
 	g_return_val_if_fail (filename != NULL, FALSE);
@@ -911,11 +919,18 @@ gedit_utils_get_glade_widgets (const gchar *filename,
 	gui = glade_xml_new (filename, root_node, NULL);
 	if (!gui)
 	{
-		msg = g_strdup_printf (GEDIT_MISSING_FILE, filename);
+		filename_markup = g_strdup_printf ("<i>%s</i>", filename);
+		msg_plain = g_strdup_printf (_("Unable to find file %s."),
+				filename_markup);
+		msg = g_strconcat ("<span size=\"large\" weight=\"bold\">",
+				msg_plain, "</span>\n\n",
+				_("Please check your installation."), NULL);
 		label = gtk_label_new (msg);
 
 		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 
+		g_free (filename_markup);
+		g_free (msg_plain);
 		g_free (msg);
 
 		*error_widget = label;
@@ -932,11 +947,19 @@ gedit_utils_get_glade_widgets (const gchar *filename,
 		*wid = glade_xml_get_widget (gui, name);
 		if (*wid == NULL)
 		{
-			msg = g_strdup_printf (GEDIT_MISSING_WIDGETS, filename);
+			filename_markup = g_strdup_printf ("<i>%s</i>", filename);
+			msg_plain = g_strdup_printf (
+					_("Unable to find the required widgets inside file %s."),
+					filename_markup);
+			msg = g_strconcat ("<span size=\"large\" weight=\"bold\">",
+					msg_plain, "</span>\n\n",
+					_("Please check your installation."), NULL);
 			label = gtk_label_new (msg);
 
 			gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 
+			g_free (filename_markup);
+			g_free (msg_plain);
 			g_free (msg);
 
 			*error_widget = label;
@@ -952,3 +975,40 @@ gedit_utils_get_glade_widgets (const gchar *filename,
 
 	return ret;
 }
+
+gchar *
+gedit_utils_make_canonical_uri_from_shell_arg (const gchar *str)
+{	
+	gchar *uri;
+	gchar *canonical_uri;
+
+	g_return_val_if_fail (str != NULL, NULL);
+	g_return_val_if_fail (*str != '\0', NULL);
+	
+	/* Note for the future: 
+	 *
+	 * <federico> paolo: and flame whoever tells 
+	 * you that file:///gnome/test_files/hëllò 
+	 * doesn't work --- that's not a valid URI
+	 *
+	 * <paolo> federico: well, another solution that 
+	 * does not requires patch to _from_shell_args 
+	 * is to check that the string returned by it 
+	 * contains only ASCII chars
+	 * <federico> paolo: hmmmm, isn't there 
+	 * gnome_vfs_is_uri_valid() or something?
+	 * <paolo>: I will use gedit_utils_is_valid_uri ()
+	 *
+	 */
+	 
+	uri = gnome_vfs_make_uri_from_shell_arg (str);
+	canonical_uri = gnome_vfs_make_uri_canonical (uri);
+	g_free (uri);
+	
+	/* g_print ("URI: %s\n", canonical_uri); */
+	
+	if (gedit_utils_is_valid_uri (canonical_uri))
+		return canonical_uri;
+	
+	return NULL;
+}
