@@ -1,19 +1,19 @@
 /*
  * @file libsexy/sexy-icon-entry.c Entry widget
  *
- * @Copyright (C) 2004-2005 Christian Hammond.
+ * @Copyright (C) 2004-2006 Christian Hammond.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
@@ -803,6 +803,8 @@ sexy_icon_entry_set_icon(SexyIconEntry *entry, SexyIconEntryPosition icon_pos,
 						 G_CALLBACK(update_icon), entry);
 
 		icon_info->icon = icon;
+
+		g_object_ref(icon);
 	}
 
 	update_icon(NULL, NULL, entry);
